# -*- coding: utf-8 -*-

from django.contrib import admin
from django_admin import AdvancedExportExcelModelAdmin, ReadOnlyModelAdmin, SpecifiedQuantityQuerySetModelAdmin

from stock.models import StockInfo, StockSendInfo
from utils.redis.connect import r
from utils.redis.rkeys import JDJOS_STOCK_CHANGED_SET
from utils.stock_utils import send_all_stock_info


class StockInfoAdmin(AdvancedExportExcelModelAdmin, admin.ModelAdmin):
    # list_display = ('stock_id', 'vendorCode', 'vendorName', 'vendorProductId', 'vendorProductName', 'storeId', 'storeName', 'quantity', 'estimateQuantity', 'inventoryDate', 'totalQuantity', 'estimateDate', 'totalEstimateQuantity', 'costPrice', 'status', 'created_at', 'updated_at')
    list_display = ('vendorProductId', 'vendorProductName', 'inventoryDate', 'totalQuantity', 'estimateDate', 'totalEstimateQuantity', 'costPrice', 'updated_at')
    readonly_fields = ('stock_id', 'vendorCode', 'vendorName', 'vendorProductId', 'vendorProductName', 'storeId', 'storeName', 'quantity', 'estimateQuantity', 'status')

    def save_model(self, request, obj, form, change):
        obj.save()
        if obj.inventoryDate and obj.estimateDate and ('inventoryDate' in form.changed_data or 'totalQuantity' in form.changed_data or 'estimateDate' in form.changed_data or 'totalEstimateQuantity' in form.changed_data or 'costPrice' in form.changed_data):
            r.sadd(JDJOS_STOCK_CHANGED_SET, obj.pk)


class StockSendInfoAdmin(SpecifiedQuantityQuerySetModelAdmin, admin.ModelAdmin):
    list_display = ('pk', 'submit')
    readonly_fields = ('status', )

    def save_model(self, request, obj, form, change):
        send_all_stock_info()


admin.site.register(StockInfo, StockInfoAdmin)
admin.site.register(StockSendInfo, StockSendInfoAdmin)
