# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django_models_ext import BaseModelMixin
from shortuuidfield import ShortUUIDField


class StockInfo(BaseModelMixin):
    stock_id = ShortUUIDField(_(u'Stock唯一标识'), max_length=32, blank=True, null=True, help_text=u'Stock唯一标识', db_index=True)

    vendorCode = models.CharField(_(u'供应商简码'), max_length=8, blank=True, null=True, help_text=u'供应商简码')
    vendorName = models.CharField(_(u'供应商名称'), max_length=32, blank=True, null=True, help_text=u'供应商名称')
    vendorProductId = models.CharField(_(u'供应商商品ID'), max_length=32, blank=True, null=True, help_text=u'供应商商品ID', db_index=True)
    vendorProductName = models.CharField(_(u'供应商商品名称'), max_length=255, blank=True, null=True, help_text=u'供应商商品名称')

    storeId = models.CharField(_(u'供应商仓库ID'), max_length=8, blank=True, null=True, help_text=u'供应商仓库ID')
    storeName = models.CharField(_(u'供应商仓库名称'), max_length=32, blank=True, null=True, help_text=u'供应商仓库名称')
    # 分仓库存数量=库存总量，预计库存数量=预计库存总量
    quantity = models.IntegerField(_(u'分仓库存数量'), default=0, help_text=u'分仓库存数量')
    estimateQuantity = models.IntegerField(_(u'预计库存数量'), default=0, help_text=u'预计库存数量')

    # Tamron 填写
    inventoryDate = models.DateTimeField(_(u'库存日期'), blank=True, null=True, help_text=_(u'库存日期'))
    totalQuantity = models.IntegerField(_(u'库存总量'), default=0, help_text=u'库存总量')
    estimateDate = models.DateTimeField(_(u'预计库存日期'), blank=True, null=True, help_text=_(u'预计库存日期'))
    totalEstimateQuantity = models.IntegerField(_(u'预计库存总量'), default=0, help_text=u'预计库存总量')
    costPrice = models.IntegerField(_(u'进价'), default=0, help_text=u'进价')

    class Meta:
        verbose_name = _(u'StockInfo')
        verbose_name_plural = _(u'StockInfo')

    def __unicode__(self):
        return u'{0.pk}'.format(self)


class StockSendInfo(BaseModelMixin):
    submit = models.BooleanField(_(u'确认提交'), default=True, help_text=_(u'确认提交'))

    class Meta:
        verbose_name = _(u'StockSendInfo')
        verbose_name_plural = _(u'StockSendInfo')

    def __unicode__(self):
        return u'{0.pk}'.format(self)
